package jp.flashcast.translator.android;

import jp.flashcast.translator.android.dao.TranslateDao;
import jp.flashcast.translator.android.model.LanguageModel;
import jp.flashcast.translator.android.model.TranslateModel;
import jp.flashcast.translator.android.model.TweetModel;
import jp.flashcast.translator.android.model.XAuthModel;
import jp.flashcast.translator.android.thread.TranslateThread;
import jp.flashcast.translator.android.thread.TweetThread;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;

public class TranslatorSample4 extends Activity implements Runnable {
    private ProgressDialog dialog;
    private TranslateModel gmodel;
    private TweetModel tmodel;
    private XAuthModel xmodel;
    private boolean translating;
    private boolean tweeting;
    private TranslateDao dao;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        initSpinners();
        initButtons();
        
        if (dao == null) {
        	dao = new TranslateDao(this);
        }
    }
    
    private void initSpinners() {
    	Spinner spSrc = (Spinner)findViewById(R.id.srclanguage);
    	Spinner spDst = (Spinner)findViewById(R.id.dstlanguage);
    	
    	String[] labels = getResources().getStringArray(R.array.language_label);
    	String[] values = getResources().getStringArray(R.array.language_value);
    	ArrayAdapter<LanguageModel> adapter = new ArrayAdapter<LanguageModel>(this, android.R.layout.simple_spinner_item);
    	adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    	
    	for (int i = 0; i < labels.length; i++) {
    		adapter.add(new LanguageModel(labels[i], values[i]));
    	}
    	
    	spSrc.setAdapter(adapter);
    	spDst.setAdapter(adapter);
    }
    
    private void initButtons() {
        // 翻訳方向ボタン
        ToggleButton btnVector = (ToggleButton)findViewById(R.id.vector);
        btnVector.setTextOff(">>");
        btnVector.setTextOn("<<");
        btnVector.setBackgroundResource(android.R.drawable.btn_default);
        
        // Tweetボタン
        Button btnTweet = (Button)findViewById(R.id.tweet);
        btnTweet.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				if (!gmodel.getTranslated().equals("")) {
					tweeting = true;
					
					if (tmodel == null) {
						tmodel = new TweetModel();
					}
					if (xmodel == null) {
						xmodel = new XAuthModel();
					}
					tmodel.setStatus(gmodel.getTranslated());
					
					Tweet();
				}
			}
        	
        });
        
        // 翻訳ボタン
        Button btnTranslate = (Button)findViewById(R.id.translate);
        btnTranslate.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
    			EditText text;
    			Spinner spSrc;
    			Spinner spDst;
    			
    			ToggleButton btnVector = (ToggleButton)findViewById(R.id.vector);
    			
    			if (btnVector.isChecked()) {
    				text = (EditText)findViewById(R.id.dsttext);
    				spSrc = (Spinner)findViewById(R.id.dstlanguage);
    				spDst = (Spinner)findViewById(R.id.srclanguage);
    			}
    			else {
    				text = (EditText)findViewById(R.id.srctext);
    				spSrc = (Spinner)findViewById(R.id.srclanguage);
    				spDst = (Spinner)findViewById(R.id.dstlanguage);
    			}
    			
    			if (!text.getText().toString().equals("")) {
    				translating = true;
    				
    				if (gmodel == null) {
    					gmodel = new TranslateModel();
    				}
    				
    				gmodel.setOriginal(text.getText().toString());
    				gmodel.setSrcLanguage(((LanguageModel)spSrc.getSelectedItem()).getValue());
    				gmodel.setDstLanguage(((LanguageModel)spDst.getSelectedItem()).getValue());
    				
    				Translate();
    			}
			}
        	
        });
    }
    
    private void setDialog(String message) {
    	if (dialog == null) {
    		dialog = new ProgressDialog(TranslatorSample4.this);
    		dialog.setIndeterminate(true);
    	}
		dialog.setMessage(message);
    	dialog.show();
    }
    
    private void Translate() {
    	setDialog("Translating... Please wait.");
    	
    	(new TranslateThread(new Handler(), TranslatorSample4.this, gmodel)).start();
    }
    
    private void Tweet() {
    	setDialog("Tweeting... Please wait.");
    	
    	(new TweetThread(new Handler(), TranslatorSample4.this, tmodel, xmodel)).start();
    }
    
	public void run() {
		dialog.dismiss();
		
		if (translating) {
			if (gmodel.isSuccess()) {
				dao.insert(gmodel);
				
				((EditText)findViewById(
					(!((ToggleButton)findViewById(R.id.vector)).isChecked()) ? R.id.dsttext : R.id.srctext))
					.setText(gmodel.getTranslated()); 
			}
			else {
				Toast.makeText(this, "Failed to translate...", Toast.LENGTH_SHORT).show();
			}
			
			translating = false;
		}
		else if (tweeting) {
			if (tmodel.isSuccess()) {
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setMessage("Tweet complete.");
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					
					public void onClick(DialogInterface dialog, int which) {
						return;
					}
					
				});
				
				builder.setCancelable(true);
				builder.create().show();
			}
			else {
				Toast.makeText(this, "Failed to tweet...", Toast.LENGTH_SHORT).show();
			}
			
			tweeting = false;
		}
	}
}